@@ -1,14 +1,18 @@
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
-#include "prefsdlg.h"
-#include "stardict.h"
+
+#include <glib/gi18n.h>
 
 #ifdef _WIN32
 #  include <gdk/gdkwin32.h>
-#  include "win32/intl.h"
 #endif
 
+#include "stardict.h"
+#include "conf.h"
+#include "utils.h"
+
+#include "prefsdlg.h"
 
 #define LOGO										0
 #define	DICTIONARY_SCAN_SETTINGS 					1
@@ -89,17 +93,8 @@ void PrefsDlg::response_handler (GtkDialog *dialog, gint res_id, PrefsDlg *oPref
 {
 	switch (res_id) {
 		case GTK_RESPONSE_HELP:
-		{
-#ifdef _WIN32
-			//gchar *filename = g_strdup_printf(_("file:///%s/help/C/stardict.html#stardict-prefs"), stardict_data_dir);
-			gchar *filename = g_strdup_printf(_("file:///%s/help/C/stardict.html"), stardict_data_dir);
-			ShellExecute((HWND)(GDK_WINDOW_HWND(gpAppFrame->oAppCore.window->window)), "OPEN", filename, NULL, NULL, SW_SHOWNORMAL);
-			g_free(filename);
-#else
-			gnome_help_display ("stardict.xml", "stardict-prefs", NULL);
-#endif
+    show_help("stardict-prefs");
 			break;
-		}
 		default:
 			oPrefsDlg->Close();
 	}
@@ -261,34 +256,19 @@ void PrefsDlg::on_setup_dictionary_scan_ckbutton_toggled(GtkToggleButton *button
 {
 	gboolean b = gtk_toggle_button_get_active (button);
 	gtk_widget_set_sensitive(oPrefsDlg->scan_modifier_key_vbox,b);
-#ifdef _WIN32
-	rw_cfg_write_boolean (usercfgfile, "preferences/dictionary", "only_scan_while_modifier_key", b);
-	on_conf_dictionary_only_scan_while_modifier_key_changed(b);
-#else
-	gpAppFrame->oAppConf.write_bool("/apps/stardict/preferences/dictionary/only_scan_while_modifier_key", b);
-#endif
+	conf->set_only_scan_while_modifier_key(b);
 }
 
 void PrefsDlg::on_setup_dictionary_scan_optionmenu_changed(GtkOptionMenu *option_menu, PrefsDlg *oPrefsDlg)
 {	
-#ifdef _WIN32	
 	gint key = gtk_option_menu_get_history(option_menu);
-	rw_cfg_write_int (usercfgfile, "preferences/dictionary", "scan_modifier_key", key);
-	on_conf_dictionary_scan_modifier_key_changed(key);
-#else
-	gpAppFrame->oAppConf.write_int("/apps/stardict/preferences/dictionary/scan_modifier_key", gtk_option_menu_get_history(option_menu));	
-#endif
+  conf->set_scan_modifier_key(key);
 }
 
 void PrefsDlg::on_setup_dictionary_scan_hide_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
 {
-#ifdef _WIN32
 	gboolean hide = gtk_toggle_button_get_active(button);
-	rw_cfg_write_boolean (usercfgfile, "preferences/dictionary", "hide_floatwin_when_modifier_key_released", hide);
-	on_conf_dictionary_hide_floatwin_when_modifier_key_released_changed(hide);
-#else
-	gpAppFrame->oAppConf.write_bool("/apps/stardict/preferences/dictionary/hide_floatwin_when_modifier_key_released", gtk_toggle_button_get_active(button));
-#endif
+  conf->set_hide_floatwin_when_modifier_key_released(hide);
 }
 
 void PrefsDlg::setup_dictionary_scan_page()
@@ -317,12 +297,8 @@ void PrefsDlg::setup_dictionary_scan_page()
 	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false, 0);
 	GtkWidget *check_button = gtk_check_button_new_with_mnemonic(_("_Only do scanning while modifier key being pressed."));
 	gtk_box_pack_start(GTK_BOX(vbox1),check_button,false,false,0);
-	gboolean only_scan_while_modifier_key;
-#ifdef _WIN32
-	rw_cfg_read_boolean (usercfgfile, "preferences/dictionary", "only_scan_while_modifier_key", &only_scan_while_modifier_key);
-#else
-	gpAppFrame->oAppConf.read_bool("/apps/stardict/preferences/dictionary/only_scan_while_modifier_key", &only_scan_while_modifier_key, false);	
-#endif
+	gboolean only_scan_while_modifier_key=conf->get_only_scan_while_modifier_key();
+
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),only_scan_while_modifier_key);
 	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_dictionary_scan_ckbutton_toggled), this);		
 	
@@ -332,12 +308,8 @@ void PrefsDlg::setup_dictionary_scan_page()
 
 	check_button = gtk_check_button_new_with_mnemonic(_("H_ide floating window when modifier key released."));
 	gtk_box_pack_start(GTK_BOX(scan_modifier_key_vbox),check_button,false,false,0);	
-	gboolean hide_floatwin_when_modifier_key_released;
-#ifdef _WIN32
-	rw_cfg_read_boolean (usercfgfile, "preferences/dictionary", "hide_floatwin_when_modifier_key_released", &hide_floatwin_when_modifier_key_released);
-#else
-	gpAppFrame->oAppConf.read_bool("/apps/stardict/preferences/dictionary/hide_floatwin_when_modifier_key_released", &hide_floatwin_when_modifier_key_released, true);	
-#endif
+	gboolean hide_floatwin_when_modifier_key_released=conf->get_hide_floatwin_when_modifier_key_released();
+
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),hide_floatwin_when_modifier_key_released);
 	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_dictionary_scan_hide_ckbutton_toggled), this);		
 
@@ -361,12 +333,8 @@ void PrefsDlg::setup_dictionary_scan_page()
 	menuitem = gtk_menu_item_new_with_mnemonic(_("_Ctrl"));
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), menu);
-	gint scan_modifier_key;
-#ifdef _WIN32
-	rw_cfg_read_int (usercfgfile, "preferences/dictionary", "scan_modifier_key", &scan_modifier_key);
-#else
-	gpAppFrame->oAppConf.read_int("/apps/stardict/preferences/dictionary/scan_modifier_key", &scan_modifier_key, 0);
-#endif
+	gint scan_modifier_key=conf->get_scan_modifier_key();
+
 	gtk_option_menu_set_history(GTK_OPTION_MENU(option_menu), scan_modifier_key);
 	
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), option_menu);
@@ -376,13 +344,9 @@ void PrefsDlg::setup_dictionary_scan_page()
 
 void PrefsDlg::on_setup_dictionary_font_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
 {
-	gboolean b = gtk_toggle_button_get_active (button);
-	gtk_widget_set_sensitive(oPrefsDlg->custom_font_hbox,b);
-#ifdef _WIN32
-	rw_cfg_write_boolean (usercfgfile, "preferences/dictionary", "use_custom_font", b);
-#else
-	gpAppFrame->oAppConf.write_bool("/apps/stardict/preferences/dictionary/use_custom_font", b);
-#endif
+  gboolean b = gtk_toggle_button_get_active(button);
+  gtk_widget_set_sensitive(oPrefsDlg->custom_font_hbox, b);
+  conf->set_use_custom_font(b);
 }
 
 void PrefsDlg::on_setup_dictionary_font_button_clicked(GtkWidget *widget, PrefsDlg *oPrefsDlg)
@@ -400,15 +364,11 @@ void PrefsDlg::on_setup_dictionary_font_button_clicked(GtkWidget *widget, PrefsD
 		font_name = gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(dlg));
 		if (font_name) {
 			gtk_button_set_label(GTK_BUTTON(widget),font_name);
-#ifdef _WIN32
-			rw_cfg_write_string (usercfgfile, "preferences/dictionary", "custom_font", font_name);
-#else
-			gpAppFrame->oAppConf.write_string("/apps/stardict/preferences/dictionary/custom_font", font_name);
-#endif
+			conf->set_custom_font(font_name);
 		}
 		break;
 		default:
-			break;
+	  /*nothing*/;
 	}
 	gtk_widget_destroy (dlg);
 }
@@ -439,12 +399,8 @@ void PrefsDlg::setup_dictionary_font_page()
 	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false, 0);
 	GtkWidget *check_button = gtk_check_button_new_with_mnemonic(_("_Use custom font."));
 	gtk_box_pack_start(GTK_BOX(vbox1),check_button,false,false,0);
-	gboolean use_custom_font;
-#ifdef _WIN32
-	rw_cfg_read_boolean (usercfgfile, "preferences/dictionary", "use_custom_font", &use_custom_font);
-#else
-	gpAppFrame->oAppConf.read_bool("/apps/stardict/preferences/dictionary/use_custom_font", &use_custom_font, false);	
-#endif
+	gboolean use_custom_font=conf->get_use_custom_font();
+
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),use_custom_font);
 	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_dictionary_font_ckbutton_toggled), this);		
 	custom_font_hbox = gtk_hbox_new(false, 12);	
@@ -455,17 +411,13 @@ void PrefsDlg::setup_dictionary_font_page()
 	gtk_box_pack_start(GTK_BOX(custom_font_hbox),label,false,false,0);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, .5);	
 	GtkWidget *button;
-	gchar *custom_font;
-#ifdef _WIN32
-	rw_cfg_read_string (usercfgfile, "preferences/dictionary", "custom_font", &custom_font);
-#else
-	gpAppFrame->oAppConf.read_string("/apps/stardict/preferences/dictionary/custom_font", &custom_font);	
-#endif
+	const gchar *custom_font=conf->get_custom_font();
+
 	if (custom_font && custom_font[0])
 		button = gtk_button_new_with_label(custom_font);
 	else
 		button=gtk_button_new_with_label(_("Choose"));	
-	g_free(custom_font);
+
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), button);
 	gtk_box_pack_start(GTK_BOX(custom_font_hbox),button,false,false,0);
 	g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (on_setup_dictionary_font_button_clicked), this);
@@ -473,13 +425,8 @@ void PrefsDlg::setup_dictionary_font_page()
 
 void PrefsDlg::on_setup_dictionary_sound_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
 {
-#ifdef _WIN32
 	gboolean enable = gtk_toggle_button_get_active(button);
-	rw_cfg_write_boolean (usercfgfile, "preferences/dictionary", "enable_sound_event", enable);
-	on_conf_dictionary_enable_sound_event_changed(enable);
-#else
-	gpAppFrame->oAppConf.write_bool("/apps/stardict/preferences/dictionary/enable_sound_event",gtk_toggle_button_get_active(button));
-#endif
+  conf->set_enable_sound_event(enable);
 }
 
 void PrefsDlg::setup_dictionary_sound_page()
@@ -510,12 +457,8 @@ void PrefsDlg::setup_dictionary_sound_page()
 	
 	GtkWidget *check_button;
 	check_button = gtk_check_button_new_with_mnemonic(_("_Enable sound event."));
-	gboolean enable;
-#ifdef _WIN32
-	rw_cfg_read_boolean (usercfgfile, "preferences/dictionary", "enable_sound_event", &enable);
-#else
-	gpAppFrame->oAppConf.read_bool("/apps/stardict/preferences/dictionary/enable_sound_event", &enable, true);
-#endif
+	gboolean enable=conf->get_enable_sound_event();
+
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),enable);
 	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_dictionary_sound_ckbutton_toggled), (gpointer)this);
 	gtk_box_pack_start(GTK_BOX(hbox1),check_button,false,false,0);
@@ -523,11 +466,7 @@ void PrefsDlg::setup_dictionary_sound_page()
 
 void PrefsDlg::on_setup_mainwin_startup_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
 {
-#ifdef _WIN32
-	rw_cfg_write_boolean (usercfgfile, "preferences/main_window", "hide_on_startup", gtk_toggle_button_get_active(button));
-#else
-	gpAppFrame->oAppConf.write_bool("/apps/stardict/preferences/main_window/hide_on_startup",gtk_toggle_button_get_active(button));
-#endif
+  conf->set_hide_on_startup(gtk_toggle_button_get_active(button));
 }
 
 void PrefsDlg::setup_mainwin_options_page()
@@ -558,12 +497,8 @@ void PrefsDlg::setup_mainwin_options_page()
 	
 	GtkWidget *check_button;
 	check_button = gtk_check_button_new_with_mnemonic(_("Hide main window when _starting StarDict."));
-	gboolean hide;
-#ifdef _WIN32
-	rw_cfg_read_boolean (usercfgfile, "preferences/main_window", "hide_on_startup", &hide);
-#else
-	gpAppFrame->oAppConf.read_bool("/apps/stardict/preferences/main_window/hide_on_startup", &hide, false);
-#endif
+	gboolean hide=conf->get_hide_on_startup();
+
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),hide);
 	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_mainwin_startup_ckbutton_toggled), (gpointer)this);
 	gtk_box_pack_start(GTK_BOX(hbox1),check_button,false,false,0);
@@ -587,14 +522,7 @@ void PrefsDlg::write_mainwin_searchwebsite_list()
 		searchwebsite_list = g_slist_append(searchwebsite_list, website);
 		have_iter = gtk_tree_model_iter_next(model, &iter);
 	}
-#ifdef _WIN32
-	rw_cfg_write_strlist (usercfgfile, "preferences/main_window", "search_website_list", searchwebsite_list);
-	on_conf_main_window_searchwebsite_list_changed(searchwebsite_list);
-#else
-	gpAppFrame->oAppConf.write_list("/apps/stardict/preferences/main_window/search_website_list", GCONF_VALUE_STRING, searchwebsite_list);
-	g_slist_foreach (searchwebsite_list, (GFunc)g_free, NULL);
-	g_slist_free(searchwebsite_list);
-#endif
+	conf->set_search_website_list(searchwebsite_list);
 }
 
 void PrefsDlg::on_setup_mainwin_searchwebsite_moveup_button_clicked(GtkWidget *widget, PrefsDlg *oPrefsDlg)
@@ -834,18 +762,13 @@ void PrefsDlg::setup_mainwin_searchwebsite_page()
 	GtkListStore *model;
 	model = gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
 
-	GSList *searchwebsite_list;
-#ifdef _WIN32
-	rw_cfg_read_strlist (usercfgfile, "preferences/main_window", "search_website_list", &searchwebsite_list);
-#else
-	gpAppFrame->oAppConf.read_list("/apps/stardict/preferences/main_window/search_website_list", GCONF_VALUE_STRING, &searchwebsite_list);
-#endif
+	const GSList *searchwebsite_list=conf->get_search_website_list();
+
 	gchar *website;
 	gchar *website_link;
 	gchar *website_searchlink;
 	GtkTreeIter iter;
-	GSList *list;
-	list = searchwebsite_list;
+	const GSList *list = searchwebsite_list;
 	while (list) {
 		website = (gchar *)(list->data);
 		website_link = strchr(website, '\t');
@@ -857,8 +780,6 @@ void PrefsDlg::setup_mainwin_searchwebsite_page()
 		}
 		list = g_slist_next(list);
 	}
-	g_slist_foreach (searchwebsite_list, (GFunc)g_free, NULL);
-	g_slist_free (searchwebsite_list);
 	
 	GtkWidget *sw;	
 	sw = gtk_scrolled_window_new (NULL, NULL);
@@ -953,13 +874,8 @@ void PrefsDlg::setup_mainwin_searchwebsite_page()
 
 void PrefsDlg::on_setup_NotificationAreaIcon_QueryInFloatWin_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
 {
-#ifdef _WIN32
 	gboolean queryin = gtk_toggle_button_get_active(button);
-	rw_cfg_write_boolean (usercfgfile, "preferences/notification_area_icon", "query_in_floatwin", queryin);
-	on_conf_notification_area_icon_show_in_floatwin_changed(queryin);
-#else
-	gpAppFrame->oAppConf.write_bool("/apps/stardict/preferences/notification_area_icon/query_in_floatwin",gtk_toggle_button_get_active(button));
-#endif
+  conf->set_query_in_floatwin(queryin);
 }
 
 void PrefsDlg::setup_NotificationAreaIcon_options_page()
@@ -990,12 +906,8 @@ void PrefsDlg::setup_NotificationAreaIcon_options_page()
 	
 	GtkWidget *check_button;
 	check_button = gtk_check_button_new_with_mnemonic(_("_Query in the floating window when middle mouse\nbutton clicked."));
-	gboolean query_in_floatwin;
-#ifdef _WIN32
-	rw_cfg_read_boolean (usercfgfile, "preferences/notification_area_icon", "query_in_floatwin", &query_in_floatwin);
-#else
-	gpAppFrame->oAppConf.read_bool("/apps/stardict/preferences/notification_area_icon/query_in_floatwin", &query_in_floatwin, true);
-#endif
+	gboolean query_in_floatwin=conf->get_query_in_floatwin();
+
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),query_in_floatwin);
 	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_NotificationAreaIcon_QueryInFloatWin_ckbutton_toggled), (gpointer)this);
 	gtk_box_pack_start(GTK_BOX(hbox1),check_button,false,false,0);
@@ -1003,13 +915,7 @@ void PrefsDlg::setup_NotificationAreaIcon_options_page()
 
 void PrefsDlg::on_setup_floatwin_pronounce_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
 {
-#ifdef _WIN32
-	gboolean pronounce = gtk_toggle_button_get_active(button);
-	rw_cfg_write_boolean (usercfgfile, "preferences/floating_window", "pronounce_when_popup", pronounce);
-	on_conf_floatwin_pronounce_when_popup_changed(pronounce);
-#else
-	gpAppFrame->oAppConf.write_bool("/apps/stardict/preferences/floating_window/pronounce_when_popup",gtk_toggle_button_get_active(button));
-#endif
+  conf->set_pronounce_when_popup(gtk_toggle_button_get_active(button));
 }
 
 void PrefsDlg::setup_floatwin_options_page()
@@ -1040,12 +946,8 @@ void PrefsDlg::setup_floatwin_options_page()
 	
 	GtkWidget *check_button;
 	check_button = gtk_check_button_new_with_mnemonic(_("_Pronouce the word when pop up."));
-	gboolean pronounce_when_popup;
-#ifdef _WIN32
-	rw_cfg_read_boolean (usercfgfile, "preferences/floating_window", "pronounce_when_popup", &pronounce_when_popup);
-#else
-	gpAppFrame->oAppConf.read_bool("/apps/stardict/preferences/floating_window/pronounce_when_popup", &pronounce_when_popup, false);
-#endif
+	gboolean pronounce_when_popup=conf->get_pronounce_when_popup();
+
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),pronounce_when_popup);
 	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_floatwin_pronounce_ckbutton_toggled), (gpointer)this);
 	gtk_box_pack_start(GTK_BOX(hbox1),check_button,false,false,0);
@@ -1053,24 +955,14 @@ void PrefsDlg::setup_floatwin_options_page()
 
 void PrefsDlg::on_setup_floatwin_size_max_width_spinbutton_changed(GtkSpinButton *button, PrefsDlg *oPrefsDlg)
 {
-#ifdef _WIN32
 	gint width = gtk_spin_button_get_value_as_int(button);
-	rw_cfg_write_int (usercfgfile, "preferences/floating_window", "max_window_width", width);
-	on_conf_floatwin_max_window_width_changed(width);
-#else
-	gpAppFrame->oAppConf.write_int("/apps/stardict/preferences/floating_window/max_window_width",gtk_spin_button_get_value_as_int(button));
-#endif
+  conf->set_max_window_width(width);
 }
 
 void PrefsDlg::on_setup_floatwin_size_max_height_spinbutton_changed(GtkSpinButton *button, PrefsDlg *oPrefsDlg)
 {
-#ifdef _WIN32
 	gint height = gtk_spin_button_get_value_as_int(button);
-	rw_cfg_write_int (usercfgfile, "preferences/floating_window", "max_window_height", height);
-	on_conf_floatwin_max_window_height_changed(height);
-#else
-	gpAppFrame->oAppConf.write_int("/apps/stardict/preferences/floating_window/max_window_height",gtk_spin_button_get_value_as_int(button));
-#endif
+  conf->set_max_window_height(height);
 }
 
 void PrefsDlg::setup_floatwin_size_page()
@@ -1099,14 +991,10 @@ void PrefsDlg::setup_floatwin_size_page()
 	vbox2 = gtk_vbox_new(false,6);
 	gtk_box_pack_start(GTK_BOX(vbox),vbox2,false,false,0);
 	
-	gint max_width,max_height;
-#ifdef _WIN32
-	rw_cfg_read_int (usercfgfile, "preferences/floating_window", "max_window_width", &max_width);
-	rw_cfg_read_int (usercfgfile, "preferences/floating_window", "max_window_height", &max_height);
-#else
-	gpAppFrame->oAppConf.read_int("/apps/stardict/preferences/floating_window/max_window_width", &max_width, DEFAULT_MAX_FLOATWIN_WIDTH);
-	gpAppFrame->oAppConf.read_int("/apps/stardict/preferences/floating_window/max_window_height", &max_height, DEFAULT_MAX_FLOATWIN_HEIGHT);
-#endif
+	gint 
+	  max_width=conf->get_max_window_width(),
+	  max_height=conf->get_max_window_height();
+
 	GdkScreen *screen = gtk_window_get_screen(GTK_WINDOW(gpAppFrame->oAppCore.window));
 	gint screen_width = gdk_screen_get_width(screen);
 	gint screen_height = gdk_screen_get_height(screen);
