@@ -25,9 +25,7 @@ struct _Skin_stardict
 	struct _Skin_cursor normal_cursor;
 	struct _Skin_cursor watch_cursor;
 	struct _Skin_pixbuf_1 icon;
-#ifdef _WIN32
-	struct _Skin_pixbuf_1 about_menu;
-#else
+#ifndef _WIN32
 	struct _Skin_pixbuf_1 docklet_normal_icon;
 	struct _Skin_pixbuf_1 docklet_scan_icon;
 	struct _Skin_pixbuf_1 docklet_stop_icon;
