@@ -1,6 +1,6 @@
 /* StarDict - A international dictionary for GNOME.
  * http://stardict.sourceforge.net
- * Copyright (C) 2003-2003 HuZheng <huzheng_001@163.com>
+ * Copyright (C) 2003-2005 HuZheng <huzheng_001@163.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  */
 
 /*
- * Modified by the StarDict Team and others 2003-2003.  See the AUTHORS
+ * Modified by the StarDict Team and others 2003-2005.  See the AUTHORS
  * file for a list of people on the StarDict Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * StarDict at http://stardict.sourceforge.net.
@@ -29,37 +29,44 @@
 #  include "config.h"
 #endif
 
-#include <string.h>
-#include <stdlib.h>
+#include <cstring>
+#include <cstdlib>
+#include <cerrno>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <glib/gi18n.h>
+#include <gdk/gdkkeysyms.h>
 
-#include "stardict.h"
-#ifndef _WIN32
-#  include "stardict-application-server.h"
-#  include "GNOME_Stardict.h"
-#endif
-#include "distance.h"
-#include "splash.h"
-
-#ifndef _WIN32
+#if !defined(_WIN32) && !defined(WITHOUT_GNOME)
 #  include <libgnome/libgnome.h>
 #  include <libgnomeui/libgnomeui.h>
 #endif
 
-#ifdef _WIN32
-	#include "win32/intl.h"
-	#include <gdk/gdkwin32.h>
-	#include <windows.h>
-	#include <gdk/gdkkeysyms.h>
-	HINSTANCE stardictexe_hInstance;
-	gchar stardict_data_dir[256];
+#if !defined(_WIN32) && !defined(WITHOUT_GNOME)
+#  include "stardict-application-server.h"
+#  include "GNOME_Stardict.h"
 #endif
 
 
+#ifdef _WIN32
+#  include <gdk/gdkwin32.h>
+#  include <windows.h>
+#  include "win32/intl.h"
+
+HINSTANCE stardictexe_hInstance;
+#endif
+
+#include "splash.h"
+#include "conf.h"
+#include "utils.h"
+
+#include "stardict.h"
+
 AppFrame * gpAppFrame;
 
 static gboolean hide_option = FALSE;
 
-#ifndef _WIN32
+#if !defined(_WIN32) && !defined(WITHOUT_GNOME)
 static gint debug = 0;
 static gboolean quit_option = FALSE;
 
@@ -82,7 +89,6 @@ static const struct poptOption options [] =
 AppCore::AppCore(AppFrame *pAppFrame)
 {
 	poAppFrame = pAppFrame;
-	iMaxFuzzyDistance  = MAX_FUZZY_DISTANCE; //need to read from cfg.
 	window = NULL; //need by save_yourself_cb().
 	prefs_dlg = NULL;
 	dict_manage_dlg = NULL;
@@ -100,23 +106,15 @@ AppCore::~AppCore()
 
 void AppCore::Create(gchar *queryword)
 {		
-	oLibs.Load();
+	oLibs.Load((gStarDictDataDir+G_DIR_SEPARATOR+"dic").c_str(), conf->get_dict_order_list(), conf->get_dict_disable_list());
 	iCurrentIndex = (glong*)g_malloc0(sizeof(glong) * oLibs.total_libs());
 
-	gboolean use_custom_font;
-#ifdef _WIN32
-	rw_cfg_read_boolean (usercfgfile, "preferences/dictionary", "use_custom_font", &use_custom_font);	
-#else
-	gpAppFrame->oAppConf.read_bool("/apps/stardict/preferences/dictionary/use_custom_font", &use_custom_font, false);
-#endif
+	gboolean use_custom_font=conf->get_use_custom_font();
+
 	if (use_custom_font)
 	{
-		gchar *custom_font;
-#ifdef _WIN32
-		rw_cfg_read_string (usercfgfile, "preferences/dictionary", "custom_font", &custom_font);
-#else
-		gpAppFrame->oAppConf.read_string("/apps/stardict/preferences/dictionary/custom_font", &custom_font);
-#endif
+		const gchar *custom_font=conf->get_custom_font();
+
 		if (custom_font && custom_font[0])
 		{
 			gchar *aa;
@@ -124,25 +122,16 @@ void AppCore::Create(gchar *queryword)
 			gtk_rc_parse_string(aa);
 			g_free(aa);
 		}
-		g_free(custom_font);
 	}
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_container_set_border_width(GTK_CONTAINER(window),2);
-	gboolean maximized;
-#ifdef _WIN32
-	rw_cfg_read_boolean (usercfgfile, "preferences/main_window", "maximized", &maximized);
-#else
-	gpAppFrame->oAppConf.read_bool("/apps/stardict/preferences/main_window/maximized", &maximized, false);
-#endif
-	gint width,height;
-#ifdef _WIN32
-	rw_cfg_read_int (usercfgfile, "preferences/main_window", "window_width", &width);
-	rw_cfg_read_int (usercfgfile, "preferences/main_window", "window_height", &height);
-#else
-	gpAppFrame->oAppConf.read_int("/apps/stardict/preferences/main_window/window_width", &width, DEFAULT_WINDOW_WIDTH);
-	gpAppFrame->oAppConf.read_int("/apps/stardict/preferences/main_window/window_height", &height, DEFAULT_WINDOW_HEIGHT);
-#endif
+	gboolean maximized=conf->get_maximized();
+
+	gint 
+	  width=conf->get_window_width(),
+	  height=conf->get_window_height();
+
 	if (width < MIN_WINDOW_WIDTH)
 		width = MIN_WINDOW_WIDTH;
 	if (height < MIN_WINDOW_HEIGHT)
@@ -179,16 +168,8 @@ void AppCore::Create(gchar *queryword)
 	oClipboard.Start();
 #endif
 
-#ifndef _WIN32
-	poAppFrame->oAppConf.EnableNotify();    
-#endif
 
-	gboolean hide;
-#ifdef _WIN32
-	rw_cfg_read_boolean (usercfgfile, "preferences/main_window", "hide_on_startup", &hide);
-#else
-	gpAppFrame->oAppConf.read_bool("/apps/stardict/preferences/main_window/hide_on_startup", &hide, false);
-#endif
+	gboolean hide=conf->get_hide_on_startup();
 	
 	//NOTICE: when docklet embedded failed,it should always show the window,but,how to detect the failure?
 	// As stardict is FOR GNOME,so i don't want to consider the case that haven't the Notification area applet.
@@ -197,7 +178,7 @@ void AppCore::Create(gchar *queryword)
 	}
 	else {
 		gdk_notify_startup_complete ();
-		if (gpAppFrame->oAppCore.oSelection.bEnable)
+		if (conf->get_scan_selection())
 			gpAppFrame->oAppCore.oDockLet.SetIcon(DOCKLET_SCAN_ICON);
 		else
 			gpAppFrame->oAppCore.oDockLet.SetIcon(DOCKLET_STOP_ICON);
@@ -236,7 +217,7 @@ gboolean AppCore::on_window_state_event(GtkWidget * window, GdkEventWindowState
 {
 	if (event->changed_mask == GDK_WINDOW_STATE_WITHDRAWN) {
 		if (event->new_window_state & GDK_WINDOW_STATE_WITHDRAWN) {
-			if (gpAppFrame->oAppCore.oSelection.bEnable) {
+			if (conf->get_scan_selection()) {
 				gpAppFrame->oAppCore.oDockLet.SetIcon(DOCKLET_SCAN_ICON);
 			}
 			else {
@@ -262,13 +243,9 @@ gboolean AppCore::on_window_state_event(GtkWidget * window, GdkEventWindowState
 			}
 		}
 	}
-	else if (event->changed_mask == GDK_WINDOW_STATE_MAXIMIZED) {
-#ifdef _WIN32
-		rw_cfg_write_boolean (usercfgfile, "preferences/main_window", "maximized", (event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED));
-#else
-		gpAppFrame->oAppConf.write_bool("/apps/stardict/preferences/main_window/maximized", (event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED));
-#endif
-	}
+	else if (event->changed_mask == GDK_WINDOW_STATE_MAXIMIZED) 
+	  conf->set_maximized((event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED));
+	
 	return false;
 }
 
@@ -301,15 +278,8 @@ gboolean AppCore::vKeyPressReleaseCallback(GtkWidget * window, GdkEventKey *even
 		}
 	}
 	else if (event->keyval==GDK_F1) {
-		if (event->type==GDK_KEY_PRESS) {
-#ifdef _WIN32
-			gchar *filename = g_strdup_printf(_("file:///%s/help/C/stardict.html"), stardict_data_dir);
-			ShellExecute((HWND)(GDK_WINDOW_HWND(gpAppFrame->oAppCore.window->window)), "OPEN", filename, NULL, NULL, SW_SHOWNORMAL);
-			g_free(filename);
-#else
-			gnome_help_display ("stardict.xml", NULL, NULL);
-#endif
-		}
+		if (event->type==GDK_KEY_PRESS)
+		  show_help(NULL);		
 	}
 	else if ((event->keyval==GDK_f || event->keyval==GDK_F) && only_ctrl_pressed) {
 		if (event->type==GDK_KEY_PRESS)
@@ -392,7 +362,7 @@ gboolean AppCore::vKeyPressReleaseCallback(GtkWidget * window, GdkEventKey *even
 	return return_val;
 }
 
-gboolean AppCore::SimpleLookupToFloat(const char* sWord,gboolean bShowIfNotFound)
+bool AppCore::SimpleLookupToFloat(const char* sWord, bool bShowIfNotFound)
 {
     if ( sWord==NULL || sWord[0]=='\0')
         return true;
@@ -449,7 +419,7 @@ gboolean AppCore::SimpleLookupToFloat(const char* sWord,gboolean bShowIfNotFound
                 ppWordData[iLib] = NULL;
 			}
         }
-        if (bFound==true)
+        if (bFound)
         {
             ShowDataToFloatWin(ppWord, ppWordData,SearchWord);
             oTopWin.InsertHisList(SearchWord);
@@ -488,13 +458,13 @@ gboolean AppCore::SimpleLookupToFloat(const char* sWord,gboolean bShowIfNotFound
     return false;
 }
 
-gboolean AppCore::SimpleLookupToTextWin(const char* sWord,glong* piIndex,gboolean piIndexValid, gboolean bTryMoreIfNotFound)
+//the input can be:
+// (sWord,NULL,false) look up the sWord.
+// (sWord,piIndex,false),look up the sWord,and set piIndex to the new indexes that found.
+// (sWord,piIndex,true), show word by piIndex's information. it will always found, so bTryMoreIfNotFound is useless.
+bool AppCore::SimpleLookupToTextWin(const char* sWord, glong* piIndex, bool piIndexValid, bool bTryMoreIfNotFound)
 {
-	//the input can be:
-	// (sWord,NULL,false) look up the sWord.
-	// (sWord,piIndex,false),look up the sWord,and set piIndex to the new indexes that found.
-	// (sWord,piIndex,true), show word by piIndex's information. it will always found, so bTryMoreIfNotFound is useless.
-	gboolean bFound = false;
+  bool bFound = false;
 	gchar **ppWord = (gchar **)g_malloc(sizeof(gchar *) * oLibs.total_libs());
 	gchar **ppWordData = (gchar **)g_malloc(sizeof(gchar *) * oLibs.total_libs());
 	glong *iIndex;
@@ -503,37 +473,35 @@ gboolean AppCore::SimpleLookupToTextWin(const char* sWord,glong* piIndex,gboolea
 	else
 		iIndex = piIndex;
 	    
-    for (int iLib=0;iLib<oLibs.total_libs();iLib++) {
+  for (int iLib=0; iLib<oLibs.total_libs(); iLib++) {
 		if (!piIndexValid) {
         	if (oLibs.LookupWord(sWord,iIndex[iLib],iLib)) {
 				ppWord[iLib] = oLibs.poGetWord(iIndex[iLib],iLib);
 				ppWordData[iLib] = oLibs.poGetWordData(iIndex[iLib],iLib);
             	bFound = true;
-        	}
-        	else {
+      } else {
 				ppWord[iLib] = NULL;
             	ppWordData[iLib] = NULL;
 			}
-		}
-		else {
-			if (piIndex[iLib] != INVALID_INDEX && (!strcmp((ppWord[iLib] = oLibs.poGetWord(piIndex[iLib],iLib)),sWord))) {				
+    } else {
+      if (piIndex[iLib] != INVALID_INDEX && 
+	  !strcmp((ppWord[iLib] = oLibs.poGetWord(piIndex[iLib],iLib)), sWord)) {
 				ppWordData[iLib] = oLibs.poGetWordData(piIndex[iLib],iLib);
 				bFound = true;
-			}
-			else {
+      }	else {
 				ppWord[iLib] = NULL;
 				ppWordData[iLib] = NULL;
 			}
 		}
     }
-	if ((!bFound)&&(!piIndexValid)) {
-		for (int iLib=0;iLib<oLibs.total_libs();iLib++) {
+
+  if (!bFound && !piIndexValid) {
+    for (int iLib=0; iLib<oLibs.total_libs(); iLib++) {
         	if (oLibs.LookupSimilarWord(sWord,iIndex[iLib],iLib)) {
 				ppWord[iLib] = oLibs.poGetWord(iIndex[iLib],iLib);
 				ppWordData[iLib] = oLibs.poGetWordData(iIndex[iLib],iLib);
             	bFound = true;
-        	}
-        	else {
+      } else {
 				ppWord[iLib] = NULL;
             	ppWordData[iLib] = NULL;
 			}
@@ -549,27 +517,24 @@ gboolean AppCore::SimpleLookupToTextWin(const char* sWord,glong* piIndex,gboolea
 			if (*hword) {
 				if (!strcmp(hword,sWord)) {
 					ShowNotFoundToTextWin(sWord,_("<Not Found!>"), TEXT_WIN_NOT_FOUND);
-				}
-				else {
+	} else {
 					for (int iLib=0;iLib<oLibs.total_libs();iLib++) {
 			        	if (oLibs.LookupWord(hword,iIndex[iLib],iLib)) {
 							ppWord[iLib] = oLibs.poGetWord(iIndex[iLib],iLib);
 							ppWordData[iLib] = oLibs.poGetWordData(iIndex[iLib],iLib);
             				bFound = true;
-        				}
-        				else {
+	    } else {
 							ppWord[iLib] = NULL;
             				ppWordData[iLib] = NULL;
 						}
 					}
 					if (!bFound) {
-						for (int iLib=0;iLib<oLibs.total_libs();iLib++) {
+	    for (int iLib=0; iLib<oLibs.total_libs(); iLib++) {
 				        	if (oLibs.LookupSimilarWord(hword,iIndex[iLib],iLib)) {
 								ppWord[iLib] = oLibs.poGetWord(iIndex[iLib],iLib);
 								ppWordData[iLib] = oLibs.poGetWordData(iIndex[iLib],iLib);
             					bFound = true;
-        					}
-				        	else {
+	      } else {
 								ppWord[iLib] = NULL;
 				            	ppWordData[iLib] = NULL;
 							}
@@ -580,13 +545,11 @@ gboolean AppCore::SimpleLookupToTextWin(const char* sWord,glong* piIndex,gboolea
 					else
 						ShowNotFoundToTextWin(sWord,_("<Not Found!>"), TEXT_WIN_NOT_FOUND);		
 				}
-			}			
-			else {
+      } else {
 				ShowNotFoundToTextWin(sWord,_("<Not Found!>"), TEXT_WIN_NOT_FOUND);
 			}
 			g_free(word);
-		}
-		else {
+    } else {
 			ShowNotFoundToTextWin(sWord,_("<Not Found!>"), TEXT_WIN_NOT_FOUND);
 		}
 	}
@@ -596,24 +559,8 @@ gboolean AppCore::SimpleLookupToTextWin(const char* sWord,glong* piIndex,gboolea
 	
 	g_free(ppWord);
 	g_free(ppWordData);
-	return bFound;
-}
 
-int AppCore::FuzzystructCompare(const void * s1, const void * s2)
-{
-    if (s1==NULL || s2==NULL)
-        return 0;
-    const struct Fuzzystruct * o1 = (struct Fuzzystruct*)s1;
-    const struct Fuzzystruct * o2 = (struct Fuzzystruct*)s2;
-    
-    if ( o1->iMatchWordDistance > o2->iMatchWordDistance )
-        return 1;
-    else if ( o1->iMatchWordDistance < o2->iMatchWordDistance )
-        return -1;
-    else if ( o1->pMatchWord && o2->pMatchWord )
-        return stardict_strcmp(o1->pMatchWord, o2->pMatchWord);
-    else
-        return 0;
+  return bFound;
 }
 
 void AppCore::LookupWithFuzzyToMainWin(const gchar *sWord)
@@ -623,113 +570,32 @@ void AppCore::LookupWithFuzzyToMainWin(const gchar *sWord)
    
 	gdk_window_set_cursor (window->window, gpAppFrame->oAppSkin.stardict.watch_cursor.cursor);
         
-    struct Fuzzystruct oFuzzystruct[MAX_FUZZY_MATCH_ITEM];
-    for (int i=0;i<MAX_FUZZY_MATCH_ITEM;i++)
-    {
-        oFuzzystruct[i].pMatchWord = NULL;
-        oFuzzystruct[i].iMatchWordDistance = iMaxFuzzyDistance;
-    }
-    int iMaxDistance = iMaxFuzzyDistance;
-    int iDistance;
-    gboolean Found = false;
-    class EditDistance oEditDistance;
-
-    glong iCheckWordLen;
-	int sCheckLen;
-	const char *sCheck;
-	gunichar *ucs4_str1,*ucs4_str2;
-	glong ucs4_str2_len;
-	char *sLowerCheckWord;
-	gchar *sLowerWord = g_utf8_strdown(sWord, -1);	
-	ucs4_str2 = g_utf8_to_ucs4_fast(sLowerWord,-1,&ucs4_str2_len);
-	g_free(sLowerWord);
-    for(int iLib=0;iLib<oLibs.total_libs();iLib++)
-    {
-        ProcessGtkEvent();
-        if((stardict_strcmp(sWord,oLibs.poGetWord(0,iLib))>=0) && (stardict_strcmp(sWord,oLibs.poGetWord(oLibs.iLength(iLib)-1,iLib))<=0)) //there are Chinese dicts and English dicts...
-        {
-            const int iwords = oLibs.iLength(iLib);
-            for (int index=0;index<iwords;index++)
-            {
-                //ProcessGtkEvent(); // too slow if here
-                sCheck = oLibs.poGetWord(index,iLib);
-                // tolower and skip too long or too short words
-				sCheckLen = strlen(sCheck);
-                iCheckWordLen = g_utf8_strlen(sCheck, sCheckLen);
-                if (iCheckWordLen-ucs4_str2_len>=iMaxDistance || ucs4_str2_len-iCheckWordLen>=iMaxDistance )
-                    continue;				
-				sLowerCheckWord = g_utf8_strdown(sCheck, sCheckLen);
-				if (iCheckWordLen > ucs4_str2_len)
-					(*g_utf8_offset_to_pointer(sLowerCheckWord, ucs4_str2_len)) = '\0';
-				ucs4_str1 = g_utf8_to_ucs4_fast(sLowerCheckWord, -1,NULL);
-				g_free(sLowerCheckWord);
-                iDistance = oEditDistance.CalEditDistance(ucs4_str1,ucs4_str2,iMaxDistance);				
-				g_free(ucs4_str1);								
-                if ( iDistance < iMaxDistance && iDistance < ucs4_str2_len) // when ucs4_str2_len=1,2 we need less fuzzy.
-                {
-                    Found = true;
-                    gboolean bAlreadyInList = false;
-                    int iMaxDistanceAt=0;
-                    for (int j=0;j<MAX_FUZZY_MATCH_ITEM;j++)
-                    {
-                      	if ( oFuzzystruct[j].pMatchWord && strcmp(oFuzzystruct[j].pMatchWord,sCheck)==0 )   //already in list
-						{
-                            bAlreadyInList = true;
-							break;
-						}
-						//find the position,it will certainly be found (include the first time) as iMaxDistance is set by last time.
-                        if ( oFuzzystruct[j].iMatchWordDistance == iMaxDistance )
-						{
-                            iMaxDistanceAt = j;
-						}
-                    }
-                    if (!bAlreadyInList )
-                    {
-						if (oFuzzystruct[iMaxDistanceAt].pMatchWord)
-							g_free(oFuzzystruct[iMaxDistanceAt].pMatchWord);
-                        oFuzzystruct[iMaxDistanceAt].pMatchWord = g_strdup(sCheck);
-                        oFuzzystruct[iMaxDistanceAt].iMatchWordDistance = iDistance;
-                        // calc new iMaxDistance
-                        iMaxDistance = iDistance;
-                        for (int j=0;j<MAX_FUZZY_MATCH_ITEM;j++)
-                        {
-                            if ( oFuzzystruct[j].iMatchWordDistance > iMaxDistance )
-                                iMaxDistance = oFuzzystruct[j].iMatchWordDistance;
-                        } // calc new iMaxDistance
-                    }   // add to list
-                }   // find one
-            }   // each word
-        }   // ok for search
-    }   // each lib
-    g_free(ucs4_str2);
+  Libs::Fuzzystruct oFuzzystruct[MAX_FUZZY_MATCH_ITEM];
+  bool Found=oLibs.LookupWithFuzzy(sWord, oFuzzystruct, 
+				   MAX_FUZZY_MATCH_ITEM, ProcessGtkEvent);
 	
     // show
     oMidWin.oIndexWin.oListWin.Clear();
     if (Found) {		
 		oMidWin.oIndexWin.oListWin.list_word_type = LIST_WIN_FUZZY_LIST;
-        // sort with distance
-        qsort(oFuzzystruct,MAX_FUZZY_MATCH_ITEM,sizeof(Fuzzystruct),FuzzystructCompare);
 
 		//SimpleLookupToTextWin(oFuzzystruct[0].pMatchWord,NULL);
 		SimpleLookupToTextWin(oFuzzystruct[0].pMatchWord, iCurrentIndex, false); // so iCurrentIndex is refreshed.
 
-        for (int i=0;i<MAX_FUZZY_MATCH_ITEM;i++) {
-            if ( oFuzzystruct[i].pMatchWord ) {
+    for (int i=0; i<MAX_FUZZY_MATCH_ITEM; i++) {
+      if (oFuzzystruct[i].pMatchWord) {
                 oMidWin.oIndexWin.oListWin.InsertLast(oFuzzystruct[i].pMatchWord);
 				g_free(oFuzzystruct[i].pMatchWord);
                 //printf("fuzzy %s,%d\n",oFuzzystruct[i].pMatchWord,oFuzzystruct[i].iMatchWordDistance);
-            }
-			else {
+      } else
 				break;
 			}
-        }
 		oMidWin.oIndexWin.oListWin.ReScroll();
-    }
-    else {
+  } else {
 		oMidWin.oIndexWin.oListWin.list_word_type = LIST_WIN_EMPTY;
         ShowNotFoundToTextWin(sWord,_("There are too many spelling errors :-("), TEXT_WIN_FUZZY_NOT_FOUND);
 	}	
-	gdk_window_set_cursor (window->window, gpAppFrame->oAppSkin.stardict.normal_cursor.cursor);
+  gdk_window_set_cursor(window->window, gpAppFrame->oAppSkin.stardict.normal_cursor.cursor);
 }
 
 void AppCore::LookupWithFuzzyToFloatWin(const gchar *sWord)
@@ -739,92 +605,16 @@ void AppCore::LookupWithFuzzyToFloatWin(const gchar *sWord)
     
 	gdk_window_set_cursor (oFloatWin.FloatWindow->window, gpAppFrame->oAppSkin.stardict.watch_cursor.cursor);
 	    
-    struct Fuzzystruct oFuzzystruct[MAX_FLOAT_WINDOW_FUZZY_MATCH_ITEM];
-    for (int i=0;i<MAX_FLOAT_WINDOW_FUZZY_MATCH_ITEM;i++)
-    {
-        oFuzzystruct[i].pMatchWord = NULL;
-        oFuzzystruct[i].iMatchWordDistance = iMaxFuzzyDistance;
-    }
-    int iMaxDistance = iMaxFuzzyDistance;
-    int iDistance;
-    gboolean Found = false;
-    class EditDistance oEditDistance;
+  Libs::Fuzzystruct oFuzzystruct[MAX_FLOAT_WINDOW_FUZZY_MATCH_ITEM];
 
-	glong iCheckWordLen;
-	int sCheckLen;
-	const char * sCheck;
-	gunichar *ucs4_str1,*ucs4_str2;
-	glong ucs4_str2_len;    
-    gchar *sLowerCheckWord;
-	gchar *sLowerWord = g_utf8_strdown(sWord, -1);
-	ucs4_str2 = g_utf8_to_ucs4_fast(sLowerWord, -1,&ucs4_str2_len);
-	g_free(sLowerWord);
-    for(int iLib=0;iLib<oLibs.total_libs();iLib++)
-    {
-        ProcessGtkEvent();
-        if((stardict_strcmp(sWord,oLibs.poGetWord(0,iLib))>=0) && (stardict_strcmp(sWord,oLibs.poGetWord(oLibs.iLength(iLib)-1,iLib))<=0)) //there are Chinese dicts and English dicts...
-        {
-            const int iwords = oLibs.iLength(iLib);
-            for (int index=0;index<iwords;index++)
-            {
-                //ProcessGtkEvent(); // too slow if here
-                sCheck = oLibs.poGetWord(index,iLib);
-                // tolower and skip too long or too short words
-				sCheckLen = strlen(sCheck);
-                iCheckWordLen = g_utf8_strlen(sCheck, sCheckLen);
-                if (iCheckWordLen-ucs4_str2_len>=iMaxDistance || ucs4_str2_len-iCheckWordLen>=iMaxDistance )
-                    continue;
-				sLowerCheckWord = g_utf8_strdown(sCheck, sCheckLen);
-				if (iCheckWordLen > ucs4_str2_len)
-					(*g_utf8_offset_to_pointer(sLowerCheckWord, ucs4_str2_len)) = '\0';
-				ucs4_str1 = g_utf8_to_ucs4_fast(sLowerCheckWord, -1,NULL);
-				g_free(sLowerCheckWord);
-                iDistance = oEditDistance.CalEditDistance(ucs4_str1,ucs4_str2,iMaxDistance);				
-				g_free(ucs4_str1);				
-                if ( iDistance < iMaxDistance && iDistance < ucs4_str2_len) // when ucs4_str2_len=1,2 we need less fuzzy.
-                {
-                    Found = true;
-                    gboolean bAlreadyInList = false;
-                    int iMaxDistanceAt=0;
-                    for (int j=0;j<MAX_FLOAT_WINDOW_FUZZY_MATCH_ITEM;j++)
-                    {
-                      	if ( oFuzzystruct[j].pMatchWord && strcmp(oFuzzystruct[j].pMatchWord,sCheck)==0 )   //already in list
-						{
-                            bAlreadyInList = true;
-							break;
-						}
-                        if ( oFuzzystruct[j].iMatchWordDistance == iMaxDistance )
-						{
-                            iMaxDistanceAt = j;
-						}
-                    }
-                    if (!bAlreadyInList )
-                    {
-						if (oFuzzystruct[iMaxDistanceAt].pMatchWord)
-							g_free(oFuzzystruct[iMaxDistanceAt].pMatchWord);
-                        oFuzzystruct[iMaxDistanceAt].pMatchWord = g_strdup(sCheck);
-                        oFuzzystruct[iMaxDistanceAt].iMatchWordDistance = iDistance;
-                        // calc new iMaxDistance
-                        iMaxDistance = iDistance;
-                        for (int j=0;j<MAX_FLOAT_WINDOW_FUZZY_MATCH_ITEM;j++)
-                        {
-                            if ( oFuzzystruct[j].iMatchWordDistance > iMaxDistance )
-                                iMaxDistance = oFuzzystruct[j].iMatchWordDistance;
-                        } // calc new iMaxDistance
-                    }   // add to list
-                }   // find one
-            }   // each word
-        }   // ok for search
-    }   // each lib
-    g_free(ucs4_str2);
+  bool Found = oLibs.LookupWithFuzzy(sWord, oFuzzystruct, 
+				     MAX_FLOAT_WINDOW_FUZZY_MATCH_ITEM,
+				     ProcessGtkEvent);
+	
+  if (Found) {
 	
-    if (Found)
-    {
-        // sort with distance
-        qsort(oFuzzystruct,MAX_FLOAT_WINDOW_FUZZY_MATCH_ITEM,sizeof(Fuzzystruct),FuzzystructCompare);
 		int i,count=0;
-        for (i=0;i<MAX_FLOAT_WINDOW_FUZZY_MATCH_ITEM;i++)
-        {
+    for (i=0;i<MAX_FLOAT_WINDOW_FUZZY_MATCH_ITEM;i++) {
             if (oFuzzystruct[i].pMatchWord)
 				count++;
 			else
@@ -836,33 +626,26 @@ void AppCore::LookupWithFuzzyToFloatWin(const gchar *sWord)
 		gchar **ppWord;
 		gchar **ppWordData;
 		glong iRetIndex;
-		for (i=0;i<count;i++)
-		{
-    		gboolean bFound = false;
+    for (i=0;i<count;i++) {
+      bool bFound = false;
 			ppWord = (gchar **)g_malloc(sizeof(gchar *) * oLibs.total_libs());
 			ppWordData = (gchar **)g_malloc(sizeof(gchar *) * oLibs.total_libs());    		
 	    
 			ppOriginWord[i] = oFuzzystruct[i].pMatchWord;
-    		for (int iLib=0;iLib<oLibs.total_libs();iLib++)
-    		{
-        		if (oLibs.SimpleLookupWord(oFuzzystruct[i].pMatchWord, iRetIndex,iLib))
-        		{
+      for (int iLib=0; iLib<oLibs.total_libs(); iLib++)	{
+	if (oLibs.SimpleLookupWord(oFuzzystruct[i].pMatchWord, iRetIndex,iLib))	{
 					ppWord[iLib] = oLibs.poGetWord(iRetIndex,iLib);
 					ppWordData[iLib] = oLibs.poGetWordData(iRetIndex,iLib);
             		bFound = true;
-        		}
-        		else {
+	} else {
 					ppWord[iLib] = NULL;
             		ppWordData[iLib] = NULL;
 				}
     		}
-    		if (bFound==true) // it is certainly be true.
-			{
+      if (bFound) {// it is certainly be true.			
 				pppWord[i]=ppWord;
 				pppWordData[i]=ppWordData;
-			}
-			else
-			{
+      }	else {
 				g_free(ppWord);
 				pppWord[i]=NULL;
 				g_free(ppWordData);
@@ -870,8 +653,7 @@ void AppCore::LookupWithFuzzyToFloatWin(const gchar *sWord)
 			}			
 		}
 		ShowDatasToFloatWin(pppWord, pppWordData, ppOriginWord, count,sWord);
-		for (i=0;i<count;i++)
-		{
+    for (i=0; i<count; i++) {
 			if (pppWord[i])
 				g_free(pppWord[i]);
 			if (pppWordData[i])
@@ -883,58 +665,22 @@ void AppCore::LookupWithFuzzyToFloatWin(const gchar *sWord)
 		
 		for (i=0;i<count;i++)
 			g_free(oFuzzystruct[i].pMatchWord);
-    }
-    else
+  } else
 		ShowNotFoundToFloatWin(sWord,_("Fuzzy query failed, too :-("), true);
-	gdk_window_set_cursor (oFloatWin.FloatWindow->window, gpAppFrame->oAppSkin.stardict.normal_cursor.cursor);
-}
-
-int AppCore::MatchWordCompare(const void * s1, const void * s2)
-{
-	const gchar **o1 = (const gchar **)s1;
-	const gchar **o2 = (const gchar **)s2;
-	return stardict_strcmp(*o1,*o2);
+  gdk_window_set_cursor(oFloatWin.FloatWindow->window, gpAppFrame->oAppSkin.stardict.normal_cursor.cursor);
 }
 
 void AppCore::LookupWithRuleToMainWin(const gchar *word)
 {
 	gdk_window_set_cursor (window->window, gpAppFrame->oAppSkin.stardict.watch_cursor.cursor);
 	
-    glong aiIndex[MAX_MATCH_ITEM_PER_LIB+1];
-    int iMatchCount = 0;
 	gchar **ppMatchWord = (gchar **)g_malloc(sizeof(gchar *) * (MAX_MATCH_ITEM_PER_LIB) * oLibs.total_libs());
-    GPatternSpec *pspec = g_pattern_spec_new(word);
-	
-    for(int iLib=0;iLib<oLibs.total_libs();iLib++)
-    {
-        //if(oLibs.LookdupWordsWithRule(pspec,aiIndex,MAX_MATCH_ITEM_PER_LIB+1-iMatchCount,iLib)) // -iMatchCount,so save time,but may got less result and the word may repeat.
-		if(oLibs.LookdupWordsWithRule(pspec,aiIndex,MAX_MATCH_ITEM_PER_LIB+1,iLib))
-        {
-            ProcessGtkEvent();
-            for(int i=0;aiIndex[i]!=-1;i++)
-            {
-                gchar * sMatchWord = oLibs.poGetWord(aiIndex[i],iLib);
-                gboolean bAlreadyInList = false;
-                for (int j=0;j<iMatchCount;j++)
-                {
-                    if ( strcmp(ppMatchWord[j],sMatchWord) == 0 )   //already in list
-                    {
-                        bAlreadyInList = true;
-                        break;
-                    }
-                }
-                if ( !bAlreadyInList)
-                    ppMatchWord[iMatchCount++] = g_strdup(sMatchWord);
-            }
-        }
-    }
-	g_pattern_spec_free(pspec);
+  gint iMatchCount=oLibs.LookupWithRule(word, ProcessGtkEvent, ppMatchWord);
 	oMidWin.oIndexWin.oListWin.Clear();
-    if (iMatchCount)
-    {		
+  if (iMatchCount) {		
 		oMidWin.oIndexWin.oListWin.list_word_type = LIST_WIN_PATTERN_LIST;
-		qsort(ppMatchWord,iMatchCount,sizeof(gchar *),MatchWordCompare); // sort it.
-        for(int i=0;i<iMatchCount;i++)
+
+    for(gint i=0; i<iMatchCount; i++)
             oMidWin.oIndexWin.oListWin.InsertLast(ppMatchWord[i]);
         //memset(iCurrentIndex,'\0',sizeof(iCurrentIndex));    // iCurrentIndex is ineffective now.
         
@@ -942,11 +688,9 @@ void AppCore::LookupWithRuleToMainWin(const gchar *word)
         //SimpleLookupToTextWin(ppMatchWord[0],NULL);
 		SimpleLookupToTextWin(ppMatchWord[0], iCurrentIndex, false); // so iCurrentIndex is refreshed.
 		oMidWin.oIndexWin.oListWin.ReScroll();
-		for(int i=0;i<iMatchCount;i++)
+    for(gint i=0; i<iMatchCount; i++)
 			g_free(ppMatchWord[i]);
-    }    
-    else
-    {
+  } else {
 		oMidWin.oIndexWin.oListWin.list_word_type = LIST_WIN_EMPTY;
         ShowNotFoundToTextWin(word,_("Found no words matching this pattern!"), TEXT_WIN_PATTERN_NOT_FOUND);
     }	
@@ -1190,23 +934,18 @@ void AppCore::End()
 	gtk_widget_destroy(window);
 }
 
-void AppCore::ProcessGtkEvent()
-{
-	poAppFrame->ProcessGtkEvent();
-}
-
 
 /***************************************************/
 AppFrame::AppFrame():oAppCore(this)
 {
-#ifndef _WIN32
+#if !defined(_WIN32) && !defined(WITHOUT_GNOME)
     gnome_sound_init(NULL);
 #endif
 }
 
 AppFrame::~AppFrame()
 {
-#ifndef _WIN32
+#if !defined(_WIN32) && !defined(WITHOUT_GNOME)
 	gnome_sound_shutdown();
 #endif
 }
@@ -1216,100 +955,48 @@ void AppFrame::Init(gchar *queryword)
 	if (!hide_option)
 		show_splash_screen();
 
-#ifdef _WIN32
-	init_conf();
-#endif
-
-#ifdef _WIN32
-	rw_cfg_read_int (usercfgfile, "preferences/dictionary", "enable_sound_event", &enable_sound_event);
-#else
-	oAppConf.read_bool("/apps/stardict/preferences/dictionary/enable_sound_event", &enable_sound_event, true);
-#endif
-	//if (enable_sound_event)
-	//	gnome_sound_play(STARDICT_DATA_DIR "/sounds/startup.wav");
-	
 	oAppSkin.load();
 		
 	oAppCore.Create(queryword);
 
-#ifndef _WIN32	
-	stardict_app_server = stardict_application_server_new (gdk_screen_get_default ());
+#if !defined(_WIN32) && !defined(WITHOUT_GNOME)
+  stardict_app_server = 
+    stardict_application_server_new(gdk_screen_get_default());
 #endif
 	
 	gtk_main();
 }
 
-void AppFrame::ProcessGtkEvent()
-{
-	while (gtk_events_pending())
-	{
-		gtk_main_iteration();
-	}
-}
-
 void AppFrame::Quit()
 {   
-#ifndef _WIN32
-	oAppConf.DisableNotify();
-#endif
+  gboolean maximized=conf->get_maximized();
 	
-	gboolean maximized;
-#ifdef _WIN32
-	rw_cfg_read_boolean (usercfgfile, "preferences/main_window", "maximized", &maximized);
-#else
-	oAppConf.read_bool("/apps/stardict/preferences/main_window/maximized", &maximized, false);
-#endif
 	if (!maximized) {
     	gint width,height;
 		gtk_window_get_size(GTK_WINDOW (oAppCore.window), &width, &height);
-#ifdef _WIN32
-		rw_cfg_write_int (usercfgfile, "preferences/main_window", "window_width", width);
-		rw_cfg_write_int (usercfgfile, "preferences/main_window", "window_height", height);
-#else
-		oAppConf.write_int("/apps/stardict/preferences/main_window/window_width",width);
-		oAppConf.write_int("/apps/stardict/preferences/main_window/window_height",height);
-#endif
+    conf->set_window_width(width);
+    conf->set_window_height(height);
+
 	}
 	gint pos = gtk_paned_get_position(GTK_PANED(oAppCore.oMidWin.hpaned));
-#ifdef _WIN32
-	rw_cfg_write_int (usercfgfile, "preferences/main_window", "hpaned_pos", pos);
-#else
-	oAppConf.write_int("/apps/stardict/preferences/main_window/hpaned_pos",pos);
-#endif
-	if (oAppCore.oFloatWin.bIsLocked) {
+  conf->set_hpaned_pos(pos);
+
+  if (conf->get_lock()) {
 		gint x,y;
 		gtk_window_get_position(GTK_WINDOW(oAppCore.oFloatWin.FloatWindow),&x,&y);
-#ifdef _WIN32
-		rw_cfg_write_int (usercfgfile, "preferences/floating_window", "lock_x", x);
-		rw_cfg_write_int (usercfgfile, "preferences/floating_window", "lock_y", y);
-#else
-		oAppConf.write_int("/apps/stardict/preferences/floating_window/lock_x",x);
-		oAppConf.write_int("/apps/stardict/preferences/floating_window/lock_y",y);
-#endif
+    conf->set_lock_x(x);
+    conf->set_lock_y(y);
 	}
     
 	oAppCore.End();
 
-#ifndef _WIN32	
+#if !defined(_WIN32) && !defined(WITHOUT_GNOME)
 	bonobo_object_unref (stardict_app_server);
 #endif
 	
-#ifdef _WIN32
-	save_conf();
-#endif
     gtk_main_quit ();
 }
 
-gint stardict_strcmp(const gchar *s1, const gchar *s2)
-{
-	gint a;
-	a = g_ascii_strcasecmp(s1, s2);
-	if (a == 0)
-		return strcmp(s1, s2);
-	else
-		return a;
-}
-
 gboolean bContainRule(const char* sWord)
 {
     int i;
@@ -1324,21 +1011,6 @@ gboolean bContainRule(const char* sWord)
     return(false);
 }
 
-gboolean bIsPureEnglish(const gchar *str) // i think this should work even when it is UTF8 string :).
-{
-    int i;
-    for(i=0;str[i]!=0;i++)
-    {
-		//use isascii()?
-		if(str[i]<0)
-        //if(str[i]<32 || str[i]>126) // tab equal 9,so this is not OK.
-        {
-            return(false);
-        }
-    }
-    return(true);	
-}
-
 gchar* GetPureEnglishAlpha(gchar *str)
 {
 	while (*str && (!((*str >= 'a' && *str <='z')||(*str >= 'A' && *str <='Z'))))
@@ -1371,19 +1043,14 @@ gchar* GetHeadWord(gchar *str)
 
 gboolean stardict_on_enter_notify (GtkWidget * widget, GdkEventCrossing * event, gpointer data)
 {
-	if (gpAppFrame->enable_sound_event && event->mode == GDK_CROSSING_NORMAL) {
-#ifdef _WIN32
-		gchar *filename = g_build_filename(stardict_data_dir, "sounds", "buttonactive.wav", NULL);
-		PlaySound(filename, 0, SND_ASYNC | SND_FILENAME);
-		g_free(filename);
-#else
-		gnome_sound_play(STARDICT_DATA_DIR "/sounds/buttonactive.wav");
-#endif
-	}
+  if (conf->get_enable_sound_event() && 
+      event->mode == GDK_CROSSING_NORMAL)
+    play_wav_file((gStarDictDataDir+G_DIR_SEPARATOR+"sounds"+G_DIR_SEPARATOR+"buttonactive.wav").c_str());
+
 	return false;
 }
 
-#ifndef _WIN32
+#if !defined(_WIN32) && !defined(WITHOUT_GNOME)
 static void
 stardict_handle_automation_cmdline (gchar *queryword)
 {
@@ -1431,6 +1098,7 @@ stardict_handle_automation_cmdline (gchar *queryword)
 static gint client_die_cb (GnomeClient *client, gpointer client_data)
 {
 	gpAppFrame->Quit();
+	return true;
 }
 
 static gint save_yourself_cb (GnomeClient       *client,
@@ -1470,33 +1138,16 @@ static gint save_yourself_cb (GnomeClient       *client,
 }
 #endif
 
-#ifdef _WIN32
-static void stardict_dummy_print( const gchar* string ) {
-	return;
+#if defined(_WIN32) || defined(WITHOUT_GNOME)
+static void stardict_dummy_print(const gchar*) 
+{       
 }
 
-static void stardict_dummy_log_handler (const gchar    *domain,
-				    GLogLevelFlags  flags,
-				    const gchar    *msg,
-				    gpointer        user_data) {
-	return;
-}
-
-static gboolean set_stardict_data_dir(void) {
-	HMODULE hmod;
-
-	hmod = GetModuleHandle(NULL);
-	if( hmod == 0 ) {
-		return true;
-	}
-	if(GetModuleFileName( hmod, (char*)&stardict_data_dir, 256 ) == 0) {
-		return true;
-	}
-	gchar* buf;
-	buf = g_path_get_dirname( stardict_data_dir );
-	strcpy( (char*)&stardict_data_dir, buf );
-	g_free( buf );
-	return false;
+static void stardict_dummy_log_handler(const gchar *,
+				       GLogLevelFlags,
+				       const gchar *,
+				       gpointer) 
+{
 }
 #endif
 
@@ -1506,13 +1157,25 @@ int stardict_main(int argc,char **argv)
 int main(int argc,char **argv)
 #endif
 {			
+  //set gStarDictDataDir;
 #ifdef _WIN32
-	if (set_stardict_data_dir())
-		return 0;
-	gchar *locale_dir;
-	locale_dir = g_strdup_printf("%s" G_DIR_SEPARATOR_S "locale", stardict_data_dir);
-	bindtextdomain (GETTEXT_PACKAGE, locale_dir);
-	g_free(locale_dir);
+  HMODULE hmod;
+
+  if ((hmod = GetModuleHandle(NULL))==0)
+    return EXIT_FAILURE;
+  char tmp_buf[256];
+  if (GetModuleFileName(hmod, tmp_buf, sizeof(tmp_buf))==0)
+    return EXIT_FAILURE;
+  
+  gchar* buf = g_path_get_dirname(tmp_buf);
+  gStarDictDataDir=buf;
+  g_free(buf);
+#else
+  gStarDictDataDir=STARDICT_DATA_DIR;
+#endif
+
+#ifdef _WIN32  
+  bindtextdomain (GETTEXT_PACKAGE, (gStarDictDataDir+G_DIR_SEPARATOR_S+"locale").c_str());
 #else
 	bindtextdomain (GETTEXT_PACKAGE, STARDICT_LOCALEDIR);
 #endif
@@ -1526,28 +1189,30 @@ int main(int argc,char **argv)
 	if (ll_winhandle > 0) {
 		SetForegroundWindow(ll_winhandle);
 		//BringWindowToTop(ll_winhandle); //don't work :(
-		//ShowWindow( ll_winhandle, SW_SHOW); //don't work fine, as gtk widgets may still be hiden.
-		return 0;
+    //don't work fine, as gtk widgets may still be hiden.
+    //ShowWindow( ll_winhandle, SW_SHOW); 
+    return EXIT_SUCCESS;
 	}
+#endif
+#if defined(_WIN32) || defined(WITHOUT_GNOME)
+  gtk_set_locale();
+  gtk_init(&argc, &argv);
 
-    gtk_set_locale ();
-    gtk_init (&argc, &argv);
-
-	g_log_set_handler (NULL, (enum GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
+  g_log_set_handler(NULL, (GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
 			   stardict_dummy_log_handler, NULL);	
-	g_log_set_handler ("Gdk", (enum GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
+  g_log_set_handler("Gdk", (GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
 			   stardict_dummy_log_handler, NULL);
-	g_log_set_handler ("Gtk", (enum GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
+  g_log_set_handler("Gtk", (GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
 			   stardict_dummy_log_handler, NULL);
-	g_log_set_handler ("GLib", (enum GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
+  g_log_set_handler("GLib", (GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
 			   stardict_dummy_log_handler, NULL);
-	g_log_set_handler ("GModule", (enum GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
+  g_log_set_handler("GModule", (GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
 			   stardict_dummy_log_handler, NULL);
-	g_log_set_handler ("GLib-GObject", (enum GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
+  g_log_set_handler("GLib-GObject", (GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
 			   stardict_dummy_log_handler, NULL);
-	g_log_set_handler ("GThread", (enum GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
+  g_log_set_handler("GThread", (GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
 			   stardict_dummy_log_handler, NULL);
-	g_set_print_handler( stardict_dummy_print );
+  g_set_print_handler(stardict_dummy_print);
 
 	gchar *queryword = NULL;
 	for (int i = 1; i < argc; i++) {
@@ -1556,13 +1221,11 @@ int main(int argc,char **argv)
 			if (strcmp(argv[i], "-h") == 0) {
 				hide_option = TRUE;
 			}
-		}
-		else {
+    } else {
 		    if (!queryword) {
 		    	if (g_utf8_validate (argv[i], -1, NULL)) {
 		    		queryword= g_strdup(argv[i]);			
-	    		}
-	    		else
+	} else
 	    			queryword = g_locale_to_utf8(argv[i],-1,NULL,NULL,NULL);
    			}
 		}
@@ -1593,12 +1256,10 @@ int main(int argc,char **argv)
 	args = (char**) poptGetArgs(pctx);
 
 	gchar *queryword = NULL;
-	if (args && args[0]) 
-	{	
+  if (args && args[0]) {	
 		//only look up the first word should OK.
-		if (g_utf8_validate (args[0], -1, NULL)) {
+    if (g_utf8_validate (args[0], -1, NULL))
 			queryword= g_strdup(args[0]);			
-		}
 		else
 			queryword = g_locale_to_utf8(args[0],-1,NULL,NULL,NULL);
 	}	
@@ -1610,14 +1271,13 @@ int main(int argc,char **argv)
 		Bonobo_ACTIVATION_FLAG_EXISTING_ONLY,
 		NULL, NULL);
 
-	if (factory != NULL) 
-	{
+  if (factory != NULL) {
 		/* there is an instance already running, so send
 		 * commands to it if needed
 		 */
                 stardict_handle_automation_cmdline (queryword);				
                 /* and we're done */
-                exit (0);
+    return EXIT_SUCCESS;
 	}
 	
 	GnomeClient *client;
@@ -1627,10 +1287,38 @@ int main(int argc,char **argv)
 	}		
 #endif		
 	
+#if defined(_WIN32) || defined(WITHOUT_GNOME)
+  gchar *conf_path;
+  gchar *conf_dir;
+# ifdef WITHOUT_GNOME
+  conf_dir = g_build_filename(g_get_home_dir(), ".stardict", NULL);
+  if (mkdir(conf_dir, S_IRWXU)==-1 && errno!=EEXIST) {
+    g_warning(N_("Can not create %s."), conf_dir);
+    g_free(conf_dir);
+    conf_dir=NULL;
+  }
+# else
+  conf_dir=g_strdup(gStarDictDataDir.c_str());
+# endif
+  conf_path = g_build_filename(conf_dir, "stardict.cfg", NULL);
+#endif
+  conf.reset(new AppConf(
+#if defined(_WIN32) || defined(WITHOUT_GNOME)
+		       conf_path
+#else
+		       "/apps/stardict"
+#endif
+		       ));
+#if defined(_WIN32) || defined(WITHOUT_GNOME)
+  g_free(conf_path);
+  g_free(conf_dir);
+#endif
+
     AppFrame oAppFrame;
 	gpAppFrame = &oAppFrame;
     oAppFrame.Init(queryword);
-	return 0;
+
+  return EXIT_SUCCESS;
 }
 
 #ifdef _WIN32
